@@ -42,6 +42,7 @@ from .libtiledb import (
     Config,
     Dim,
     Domain,
+    Attr,
     ArraySchema,
     consolidate,
     object_type,
@@ -58,8 +59,6 @@ from .libtiledb import (
 
 from .array import DenseArray, SparseArray
 
-from .attribute import Attr
-
 from .filter import (
     Filter,
     FilterList,
@@ -71,6 +70,7 @@ from .filter import (
     RleFilter,
     DoubleDeltaFilter,
     DictionaryFilter,
+    XORFilter,
     BitShuffleFilter,
     ByteShuffleFilter,
     BitWidthReductionFilter,
