@@ -21,20 +21,18 @@ void init_enums(py::module &m) {
   m.def("TILEDB_TIMESTAMP_NOW_MS", []() { return TILEDB_TIMESTAMP_NOW_MS; });
 
   py::enum_<tiledb_datatype_t>(m, "DataType", py::module_local()) DENUM(
-      INT32) DENUM(INT64) DENUM(FLOAT32) DENUM(FLOAT64) DENUM(CHAR) DENUM(INT8)
-      DENUM(UINT8) DENUM(INT16) DENUM(UINT16) DENUM(UINT32) DENUM(UINT64) DENUM(
-          BOOL) DENUM(STRING_ASCII) DENUM(STRING_UTF8) DENUM(STRING_UTF16)
+      INT32) DENUM(INT64) DENUM(FLOAT32) DENUM(FLOAT64) DENUM(CHAR) DENUM(BLOB)
+      DENUM(INT8) DENUM(UINT8) DENUM(INT16) DENUM(UINT16) DENUM(UINT32) DENUM(
+          UINT64) DENUM(STRING_ASCII) DENUM(STRING_UTF8) DENUM(STRING_UTF16)
           DENUM(STRING_UTF32) DENUM(STRING_UCS2) DENUM(STRING_UCS4) DENUM(ANY)
-              DENUM(DATETIME_YEAR) DENUM(DATETIME_MONTH) DENUM(DATETIME_WEEK)
-                  DENUM(DATETIME_DAY) DENUM(DATETIME_HR) DENUM(DATETIME_MIN)
-                      DENUM(DATETIME_SEC) DENUM(DATETIME_MS) DENUM(DATETIME_US)
-                          DENUM(DATETIME_NS) DENUM(DATETIME_PS)
-                              DENUM(DATETIME_FS) DENUM(DATETIME_AS)
-                                  DENUM(TIME_HR) DENUM(TIME_MIN) DENUM(TIME_SEC)
-                                      DENUM(TIME_MS) DENUM(TIME_US)
-                                          DENUM(TIME_NS) DENUM(TIME_PS)
-                                              DENUM(TIME_FS) DENUM(TIME_AS)
-                                                  DENUM(BLOB);
+              DENUM(DATETIME_YEAR) DENUM(DATETIME_WEEK) DENUM(DATETIME_DAY)
+                  DENUM(DATETIME_HR) DENUM(DATETIME_MIN) DENUM(DATETIME_SEC)
+                      DENUM(DATETIME_MS) DENUM(DATETIME_US) DENUM(DATETIME_NS)
+                          DENUM(DATETIME_PS) DENUM(DATETIME_FS)
+                              DENUM(DATETIME_AS) DENUM(TIME_HR) DENUM(TIME_MIN)
+                                  DENUM(TIME_SEC) DENUM(TIME_MS) DENUM(TIME_US)
+                                      DENUM(TIME_NS) DENUM(TIME_PS)
+                                          DENUM(TIME_FS) DENUM(TIME_AS);
 
   py::enum_<tiledb_array_type_t>(m, "ArrayType") DENUM(DENSE) DENUM(SPARSE);
 
@@ -46,8 +44,8 @@ void init_enums(py::module &m) {
       DFENUM(ZSTD) DFENUM(LZ4) DFENUM(RLE) DFENUM(BZIP2) DFENUM(DOUBLE_DELTA)
           DFENUM(BIT_WIDTH_REDUCTION) DFENUM(BITSHUFFLE) DFENUM(BYTESHUFFLE)
               DFENUM(POSITIVE_DELTA) DFENUM(CHECKSUM_MD5)
-                  DFENUM(CHECKSUM_SHA256) DFENUM(DICTIONARY)
-                      DFENUM(SCALE_FLOAT);
+                  DFENUM(CHECKSUM_SHA256) DFENUM(SCALE_FLOAT) DFENUM(DICTIONARY)
+                      DFENUM(XOR);
 
   py::enum_<tiledb_filter_option_t>(m, "FilterOption") DENUM(COMPRESSION_LEVEL)
       DENUM(BIT_WIDTH_MAX_WINDOW) DENUM(POSITIVE_DELTA_MAX_WINDOW)
@@ -65,7 +63,11 @@ void init_enums(py::module &m) {
       .value("UNINITIALIZED", Query::Status::UNINITIALIZED)
       .export_values();
 
-  py::enum_<tiledb_query_type_t>(m, "QueryType") DENUM(READ) DENUM(WRITE);
+  py::enum_<tiledb_query_type_t>(m, "QueryType")
+      .value("READ", TILEDB_READ)
+      .value("WRITE", TILEDB_WRITE)
+      .value("DELETE", TILEDB_DELETE)
+      .value("MODIFY_EXCLUSIVE", TILEDB_MODIFY_EXCLUSIVE);
 
   py::enum_<tiledb_query_condition_op_t>(m, "QueryConditionOp",
                                          py::module_local()) DENUM(LT) DENUM(LE)
