@@ -15,9 +15,6 @@
 #include <pybind11/pytypes.h>
 #include <pybind11/stl.h>
 
-#define TILEDB_DEPRECATED
-#define TILEDB_DEPRECATED_EXPORT
-
 #include <tiledb/tiledb.h>              // C
 #include <tiledb/tiledb>                // C++
 #include <tiledb/tiledb_experimental.h> // C
@@ -358,6 +355,16 @@ public:
 
     bool issparse = array_->schema().array_type() == TILEDB_SPARSE;
 
+    std::string mode = py::str(array.attr("mode"));
+    auto query_mode = TILEDB_READ;
+    if (mode == "r") {
+      query_mode = TILEDB_READ;
+    } else if (mode == "d") {
+      query_mode = TILEDB_DELETE;
+    } else {
+      throw std::invalid_argument("Invalid query mode: " + mode);
+    }
+
     // initialize the dims that we are asked to read
     for (auto d : dims) {
       dims_.push_back(d.cast<string>());
@@ -368,32 +375,36 @@ public:
       attrs_.push_back(a.cast<string>());
     }
 
-    py::object pre_buffers = array.attr("_buffers");
-    if (!pre_buffers.is(py::none())) {
-      py::dict pre_buffers_dict = pre_buffers.cast<py::dict>();
+    if (query_mode == TILEDB_READ) {
+      py::object pre_buffers = array.attr("_buffers");
+      if (!pre_buffers.is(py::none())) {
+        py::dict pre_buffers_dict = pre_buffers.cast<py::dict>();
 
-      // iterate over (key, value) pairs
-      for (std::pair<py::handle, py::handle> b : pre_buffers_dict) {
-        py::str name = b.first.cast<py::str>();
+        // iterate over (key, value) pairs
+        for (std::pair<py::handle, py::handle> b : pre_buffers_dict) {
+          py::str name = b.first.cast<py::str>();
 
-        // unpack value tuple of (data, offsets)
-        auto bfrs = b.second.cast<std::pair<py::handle, py::handle>>();
-        auto data_array = bfrs.first.cast<py::array>();
-        auto offsets_array = bfrs.second.cast<py::array>();
+          // unpack value tuple of (data, offsets)
+          auto bfrs = b.second.cast<std::pair<py::handle, py::handle>>();
+          auto data_array = bfrs.first.cast<py::array>();
+          auto offsets_array = bfrs.second.cast<py::array>();
 
-        import_buffer(name, data_array, offsets_array);
+          import_buffer(name, data_array, offsets_array);
+        }
       }
     }
 
     query_ =
-        std::shared_ptr<tiledb::Query>(new Query(ctx_, *array_, TILEDB_READ));
+        std::shared_ptr<tiledb::Query>(new Query(ctx_, *array_, query_mode));
     //        [](Query* p){} /* note: no deleter*/);
 
-    layout_ = (tiledb_layout_t)py_layout.cast<int32_t>();
-    if (!issparse && layout_ == TILEDB_UNORDERED) {
-      TPY_ERROR_LOC("TILEDB_UNORDERED read is not supported for dense arrays")
+    if (query_mode == TILEDB_READ) {
+      layout_ = (tiledb_layout_t)py_layout.cast<int32_t>();
+      if (!issparse && layout_ == TILEDB_UNORDERED) {
+        TPY_ERROR_LOC("TILEDB_UNORDERED read is not supported for dense arrays")
+      }
+      query_->set_layout(layout_);
     }
-    query_->set_layout(layout_);
 
 #if TILEDB_VERSION_MAJOR >= 2 && TILEDB_VERSION_MINOR >= 2
     if (use_arrow_) {
@@ -706,22 +717,19 @@ public:
   }
 #endif
 
-  void set_attr_cond(py::object attr_cond) {
-    if (!attr_cond.is(py::none())) {
-      py::object init_pyqc = attr_cond.attr("init_query_condition");
+  void set_cond(py::object cond) {
+    py::object init_pyqc = cond.attr("init_query_condition");
 
-      try {
-        attrs_ = init_pyqc(pyschema_, attrs_).cast<std::vector<std::string>>();
-      } catch (tiledb::TileDBError &e) {
-        TPY_ERROR_LOC(e.what());
-      } catch (py::error_already_set &e) {
-        TPY_ERROR_LOC(e.what());
-      }
-
-      auto pyqc = (attr_cond.attr("c_obj")).cast<PyQueryCondition>();
-      auto qc = pyqc.ptr().get();
-      query_->set_condition(*qc);
+    try {
+      attrs_ = init_pyqc(pyschema_, attrs_).cast<std::vector<std::string>>();
+    } catch (tiledb::TileDBError &e) {
+      TPY_ERROR_LOC(e.what());
+    } catch (py::error_already_set &e) {
+      TPY_ERROR_LOC(e.what());
     }
+    auto pyqc = (cond.attr("c_obj")).cast<PyQueryCondition>();
+    auto qc = pyqc.ptr().get();
+    query_->set_condition(*qc);
   }
 
   bool is_dimension(std::string name) { return domain_->has_dimension(name); }
@@ -793,9 +801,6 @@ public:
     tiledb_datatype_t type;
     uint32_t cell_val_num;
     std::tie(type, cell_val_num) = buffer_type(name);
-    uint64_t cell_nbytes = tiledb_datatype_size(type);
-    if (cell_val_num != TILEDB_VAR_NUM)
-      cell_nbytes *= cell_val_num;
     auto dtype = tiledb_dtype(type, cell_val_num);
 
     buffers_order_.push_back(name);
@@ -980,6 +985,20 @@ public:
       buf.data_vals_read += data_vals_num;
       buf.offsets_read += offset_elem_num;
       buf.validity_vals_read += validity_elem_num;
+
+      if ((Py_ssize_t)(buf.data_vals_read * buf.elem_nbytes) >
+          (Py_ssize_t)buf.data.size()) {
+        throw TileDBError("After read query, data buffer out of bounds: " +
+                          name);
+      }
+      if ((Py_ssize_t)buf.offsets_read > buf.offsets.size()) {
+        throw TileDBError("After read query, offsets buffer out of bounds: " +
+                          name);
+      }
+      if ((Py_ssize_t)buf.validity_vals_read > buf.validity.size()) {
+        throw TileDBError("After read query, validity buffer out of bounds: " +
+                          name);
+      }
     }
   }
 
@@ -1347,6 +1366,8 @@ public:
       submit_read();
     else if (array_->query_type() == TILEDB_WRITE)
       submit_write();
+    else if (array_->query_type() == TILEDB_DELETE)
+      query_->submit();
     else
       TPY_ERROR_LOC("Unknown query type!")
   }
@@ -1608,7 +1629,7 @@ void init_core(py::module &m) {
           .def("set_ranges_bulk", &PyQuery::set_ranges_bulk)
 #endif
           .def("set_subarray", &PyQuery::set_subarray)
-          .def("set_attr_cond", &PyQuery::set_attr_cond)
+          .def("set_cond", &PyQuery::set_cond)
 #if defined(TILEDB_SERIALIZATION)
           .def("set_serialized_query", &PyQuery::set_serialized_query)
 #endif
