@@ -1,4 +1,5 @@
 import io
+import numpy as np
 from typing import List, overload, Sequence, TYPE_CHECKING, Union
 
 import tiledb.cc as lt
@@ -13,7 +14,8 @@ class Filter(lt.Filter):
 
     def __init__(self, type: lt.FilterOption, ctx: "Ctx" = None):
         self._ctx = ctx or default_ctx()
-        super().__init__(lt.Context(self._ctx, False), type)
+
+        super().__init__(self._ctx, type)
 
     def __repr__(self) -> str:
         output = io.StringIO()
@@ -82,17 +84,14 @@ class CompressionFilter(Filter):
 
         super().__init__(type, self._ctx)
         self._set_option(
-            lt.Context(self._ctx, False),
+            self._ctx,
             lt.FilterOption.COMPRESSION_LEVEL,
             level,
         )
 
     @property
     def level(self):
-        return self._get_option(
-            lt.Context(self._ctx.__capsule__(), False),
-            lt.FilterOption.COMPRESSION_LEVEL,
-        )
+        return self._get_option(self._ctx, lt.FilterOption.COMPRESSION_LEVEL)
 
 
 class NoOpFilter(Filter):
@@ -407,7 +406,7 @@ class BitWidthReductionFilter(Filter):
 
         if window != -1:
             self._set_option(
-                lt.Context(self._ctx.__capsule__(), False),
+                self._ctx,
                 lt.FilterOption.BIT_WIDTH_MAX_WINDOW,
                 window,
             )
@@ -417,10 +416,7 @@ class BitWidthReductionFilter(Filter):
 
     @property
     def window(self):
-        return self._get_option(
-            lt.Context(self._ctx.__capsule__(), False),
-            lt.FilterOption.BIT_WIDTH_MAX_WINDOW,
-        )
+        return self._get_option(self._ctx, lt.FilterOption.BIT_WIDTH_MAX_WINDOW)
 
 
 class PositiveDeltaFilter(Filter):
@@ -451,7 +447,7 @@ class PositiveDeltaFilter(Filter):
         super().__init__(lt.FilterType.POSITIVE_DELTA)
         if window != -1:
             self._set_option(
-                lt.Context(self._ctx.__capsule__(), False),
+                self._ctx,
                 lt.FilterOption.POSITIVE_DELTA_MAX_WINDOW,
                 window,
             )
@@ -461,10 +457,7 @@ class PositiveDeltaFilter(Filter):
 
     @property
     def window(self):
-        return self._get_option(
-            lt.Context(self._ctx.__capsule__(), False),
-            lt.FilterOption.POSITIVE_DELTA_MAX_WINDOW,
-        )
+        return self._get_option(self._ctx, lt.FilterOption.POSITIVE_DELTA_MAX_WINDOW)
 
 
 class ChecksumMD5Filter(Filter):
@@ -520,14 +513,11 @@ class FloatScaleFilter(Filter):
     Filter that stores floats as integers in a reduced representation via scaling.
     The reduced storage space is in lieu of some precision loss. The float scaling
     filter takes three parameters: the scale, the offset, and the byte width.
-
     On write, the float scaling filter applies the scale factor and offset,
     and stores the value of round((raw_float - offset) / scale) as an
     integer with the specified NumPy dtype.
-
     On read, the float scaling filter will reverse the scale factor and offset,
     and returns the floating point data, with a potential loss of precision.
-
     :param factor: the scaling factor used to translate the data
     :type factor: float
     :param offset: the offset value used to translate the data
@@ -536,9 +526,7 @@ class FloatScaleFilter(Filter):
     :type np.integer:
     :param ctx: A TileDB Context
     :type ctx: tiledb.Ctx
-
     **Example:**
-
     >>> import tiledb, numpy as np, tempfile
     >>> with tempfile.TemporaryDirectory() as tmp:
     ...     dom = tiledb.Domain(tiledb.Dim(domain=(0, 9), tile=2, dtype=np.uint64))
@@ -546,7 +534,6 @@ class FloatScaleFilter(Filter):
     ...                      filters=tiledb.FilterList([tiledb.FloatScaleFilter(1, 0)]))
     ...     schema = tiledb.ArraySchema(domain=dom, attrs=(a1,))
     ...     tiledb.DenseArray.create(tmp + "/array", schema)
-
     """
 
     def __init__(
@@ -565,7 +552,7 @@ class FloatScaleFilter(Filter):
 
         if factor:
             self._set_option(
-                lt.Context(self._ctx.__capsule__(), False),
+                self._ctx,
                 lt.FilterOption.SCALE_FLOAT_FACTOR,
                 float(factor),
             )
@@ -574,21 +561,21 @@ class FloatScaleFilter(Filter):
             # 0 evals to false so we weren't actually setting here
             # if offset==0 :upside-down-face:
             self._set_option(
-                lt.Context(self._ctx.__capsule__(), False),
+                self._ctx,
                 lt.FilterOption.SCALE_FLOAT_OFFSET,
                 float(offset),
             )
 
         if bytewidth:
             self._set_option(
-                lt.Context(self._ctx.__capsule__(), False),
+                self._ctx,
                 lt.FilterOption.SCALE_FLOAT_BYTEWIDTH,
                 bytewidth,
             )
 
     def dump(self):
         self._dump(
-            lt.Context(self._ctx.__capsule__(), False),
+            self._ctx,
         )
 
     def _attrs_(self):
@@ -600,27 +587,41 @@ class FloatScaleFilter(Filter):
 
     @property
     def factor(self):
-        return self._get_option(
-            lt.Context(self._ctx.__capsule__(), False),
-            lt.FilterOption.SCALE_FLOAT_FACTOR,
-        )
+        return self._get_option(self._ctx, lt.FilterOption.SCALE_FLOAT_FACTOR)
 
     @property
     def offset(self):
-        return self._get_option(
-            lt.Context(self._ctx.__capsule__(), False),
-            lt.FilterOption.SCALE_FLOAT_OFFSET,
-        )
+        return self._get_option(self._ctx, lt.FilterOption.SCALE_FLOAT_OFFSET)
 
     @property
     def bytewidth(self):
-        return self._get_option(
-            lt.Context(self._ctx.__capsule__(), False),
-            lt.FilterOption.SCALE_FLOAT_BYTEWIDTH,
-        )
+        return self._get_option(self._ctx, lt.FilterOption.SCALE_FLOAT_BYTEWIDTH)
+
+
+class XORFilter(Filter):
+    """
+    XOR  filter.
+
+    **Example:**
+
+    >>> import tiledb, numpy as np, tempfile
+    >>> with tempfile.TemporaryDirectory() as tmp:
+    ...     dom = tiledb.Domain(tiledb.Dim(domain=(0, 9), tile=2, dtype=np.uint64))
+    ...     a1 = tiledb.Attr(name="a1", dtype=np.int64,
+    ...                      filters=tiledb.FilterList([tiledb.XORFilter()]))
+    ...     schema = tiledb.ArraySchema(domain=dom, attrs=(a1,))
+    ...     tiledb.DenseArray.create(tmp + "/array", schema)
+
+    """
+
+    def __init__(self, ctx: "Ctx" = None):
+        self._ctx = ctx or default_ctx()
+        super().__init__(lt.FilterType.XOR, self._ctx)
+
+    def _attrs_(self):
+        return {}
 
 
-#
 class FilterList(lt.FilterList):
     """
     An ordered list of Filter objects for filtering TileDB data.
@@ -667,6 +668,7 @@ class FilterList(lt.FilterList):
         lt.FilterType.CHECKSUM_SHA256: ChecksumSHA256Filter,
         lt.FilterType.DICTIONARY: DictionaryFilter,
         lt.FilterType.SCALE_FLOAT: FloatScaleFilter,
+        lt.FilterType.XOR: XORFilter,
         lt.FilterType.NONE: NoOpFilter,
     }
 
@@ -675,21 +677,16 @@ class FilterList(lt.FilterList):
         filters: Sequence[Filter] = None,
         chunksize: int = None,
         ctx: "Ctx" = None,
-        _lt_obj=None,
-        _capsule=None,
+        is_capsule: bool = False,
     ):
         self._ctx = ctx or default_ctx()
-        _cctx = lt.Context(self._ctx, False)
+        _cctx = self._ctx
 
-        if _capsule is not None:
-            super().__init__(_cctx, _capsule)
-        elif _lt_obj is not None:
-            super().__init__(_cctx)
-            for i in range(_lt_obj._nfilters()):
-                self._add_filter(_lt_obj._filter(i))
-            chunksize = _lt_obj._chunksize
+        if is_capsule:
+            super().__init__(_cctx, filters)
         else:
             super().__init__(_cctx)
+
             if filters is not None:
                 filters = list(filters)
                 for f in filters:
@@ -815,7 +812,6 @@ class FilterList(lt.FilterList):
     def _getfilter(self, i: int) -> Filter:
         fil = self._filter(i)
         filtype = self.filter_type_cc_to_python[fil._type]
-        options = None
 
         if issubclass(filtype, CompressionFilter):
             options = [lt.FilterOption.COMPRESSION_LEVEL]
@@ -829,14 +825,15 @@ class FilterList(lt.FilterList):
                 lt.FilterOption.SCALE_FLOAT_OFFSET,
                 lt.FilterOption.SCALE_FLOAT_BYTEWIDTH,
             ]
+        else:
+            options = []
 
-        if options is not None:
-            _cctx = lt.Context(self._ctx.__capsule__(), False)
+        _cctx = self._ctx
 
-            opts = []
-            for opt in options:
-                opts.append(fil._get_option(_cctx, opt))
+        opts = []
+        for opt in options:
+            opts.append(fil._get_option(_cctx, opt))
 
-            filter = filtype(*opts, ctx=self._ctx)
+        fil = filtype(*opts, ctx=self._ctx)
 
-        return filter
+        return fil
