@@ -16,37 +16,6 @@ typedef struct {
 
 using ArraySchemaEvolution = PyArraySchemaEvolution;
 
-void _throw_tiledb_error(tiledb_error_t *err_ptr) {
-  const char *err_msg_ptr = NULL;
-  int ret = tiledb_error_message(err_ptr, &err_msg_ptr);
-  if (ret != TILEDB_OK) {
-    tiledb_error_free(&err_ptr);
-    if (ret == TILEDB_OOM) {
-      throw std::bad_alloc();
-    }
-    TPY_ERROR_LOC("error retrieving error message");
-  }
-
-  TPY_ERROR_LOC(std::string(err_msg_ptr));
-}
-
-void _throw_ctx_err(tiledb_ctx_t *ctx_ptr, int rc) {
-  if (rc == TILEDB_OK)
-    return;
-  if (rc == TILEDB_OOM)
-    throw std::bad_alloc();
-
-  tiledb_error_t *err_ptr = NULL;
-  int ret = tiledb_ctx_get_last_error(ctx_ptr, &err_ptr);
-  if (ret != TILEDB_OK) {
-    tiledb_error_free(&err_ptr);
-    if (ret == TILEDB_OOM)
-      throw std::bad_alloc();
-    TPY_ERROR_LOC("error retrieving error object from ctx");
-  }
-  _throw_tiledb_error(err_ptr);
-}
-
 void init_schema_evolution(py::module &m) {
   py::class_<ArraySchemaEvolution>(m, "ArraySchemaEvolution")
       .def(py::init([](py::object ctx_py) {
@@ -57,7 +26,7 @@ void init_schema_evolution(py::module &m) {
         tiledb_array_schema_evolution_t *evol_p;
         int rc = tiledb_array_schema_evolution_alloc(ctx_c, &evol_p);
         if (rc != TILEDB_OK) {
-          _throw_ctx_err(ctx_c, rc);
+          TPY_ERROR_LOC(get_last_ctx_err_str(ctx_c, rc));
         }
 
         return new PyArraySchemaEvolution({ctx_c, evol_p});
@@ -72,7 +41,7 @@ void init_schema_evolution(py::module &m) {
              int rc = tiledb_array_schema_evolution_add_attribute(
                  inst.ctx_, inst.evol_, attr_c);
              if (rc != TILEDB_OK) {
-               _throw_ctx_err(inst.ctx_, rc);
+               TPY_ERROR_LOC(get_last_ctx_err_str(inst.ctx_, rc));
              }
            })
       .def("drop_attribute",
@@ -80,13 +49,13 @@ void init_schema_evolution(py::module &m) {
              int rc = tiledb_array_schema_evolution_drop_attribute(
                  inst.ctx_, inst.evol_, attr_name.c_str());
              if (rc != TILEDB_OK) {
-               _throw_ctx_err(inst.ctx_, rc);
+               TPY_ERROR_LOC(get_last_ctx_err_str(inst.ctx_, rc));
              }
            })
       .def("array_evolve", [](ArraySchemaEvolution &inst, std::string uri) {
         int rc = tiledb_array_evolve(inst.ctx_, uri.c_str(), inst.evol_);
         if (rc != TILEDB_OK) {
-          _throw_ctx_err(inst.ctx_, rc);
+          TPY_ERROR_LOC(get_last_ctx_err_str(inst.ctx_, rc));
         }
       });
 }
